@@ -49,10 +49,18 @@ class LLMProvider(Enum):
     GEMINI = "gemini"
 
 
+class Origin(Enum):
+    DEBUGGER = "debugger"
+    WEB = "web"
+    PHONE = "phone"
+    UNKNOWN = "unknown"
+
+
 class Configuration:
     def __init__(
         self,
         input_type: str = "mic",
+        origin: Origin = Origin.UNKNOWN,
         default_voice: str = "nova",
         custom_voices: Mapping[str, CustomVoice] = {},
         llm_provider: LLMProvider = LLMProvider.OPENAI,
@@ -79,7 +87,8 @@ class Configuration:
         error_fallback_timeout_seconds: int = 0,
         error_fallback_occurrence_threshold: int = 5,
         error_fallback_time_window_seconds: int = 300,
-        ringing_enabled: bool = False,
+        startup_audio_url: str = None,
+        startup_audio_interruptible: bool = False,
     ):
         self.input_type = input_type
         self.default_voice = default_voice
@@ -110,19 +119,26 @@ class Configuration:
         self.error_fallback_timeout_seconds = error_fallback_timeout_seconds
         self.error_fallback_occurrence_threshold = error_fallback_occurrence_threshold
         self.error_fallback_time_window_seconds = error_fallback_time_window_seconds
-        self.ringing_enabled = ringing_enabled
+        self.startup_audio_url = startup_audio_url
+        self.startup_audio_interruptible = startup_audio_interruptible
 
         if isinstance(llm_provider, str):
             self.llm_provider = LLMProvider(llm_provider)
         else:
             self.llm_provider = llm_provider
 
+        if isinstance(origin, str):
+            self.origin = Origin(origin)
+        else:
+            self.origin = origin
+
     def set_llm_provider(self, llm_provider: LLMProvider):
         self.llm_provider = llm_provider
 
     def __dict__(self):
         return {
             "input_type": self.input_type,
+            "origin": self.origin.value,
             "default_voice": self.default_voice,
             "custom_voices": {k: v.__dict__() for k, v in self.custom_voices.items()},
             "llm_provider": self.llm_provider.value,
@@ -149,5 +165,6 @@ class Configuration:
             "error_fallback_timeout_seconds": self.error_fallback_timeout_seconds,
             "error_fallback_occurrence_threshold": self.error_fallback_occurrence_threshold,
             "error_fallback_time_window_seconds": self.error_fallback_time_window_seconds,
-            "ringing_enabled": self.ringing_enabled,
+            "startup_audio_url": self.startup_audio_url,
+            "startup_audio_interruptible": self.startup_audio_interruptible,
         }
