@@ -1,8 +1,13 @@
-from typing import List, Dict, Any
+import asyncio
 import json
+import uuid
+
+from typing import List, Dict, Any, Callable, Coroutine
 
 from .configuration import Configuration, LLMProvider
+from .events import Event, ErrorEvent
 from .testing import Testing, Test, Outcome
+from .utils.errors import format_error_with_filtered_traceback
 
 
 class Context:
@@ -14,6 +19,8 @@ class Context:
         configuration: Configuration = Configuration(),
         data: dict = {},
         variables: Dict[str, float] = {},
+        on_event: Callable[[Event], None] = lambda: None,
+        on_debug_event: Callable[[Event, str, dict], None] = lambda: None,
     ):
         self.agent_id = agent_id
         self.environment = environment
@@ -24,6 +31,10 @@ class Context:
         self.history = []
         self.testing = Testing(runTest=json.loads(self.data.get("runTest", "{}")))
         self.function_id = self.data.get("functionId", "")
+        self.on_event = on_event
+        self.on_debug_event = on_debug_event
+        self.tasks: Dict[str, asyncio.Task] = {}
+
         if "runTest" in self.data:
             del self.data["runTest"]  # Remove runTest from data
         if "functionId" in self.data:
@@ -263,6 +274,34 @@ class Context:
         """
         self.history.append({"role": "user", "content": message})
 
+    def add_tool_message(self, tool_name: str, result: str, tool_call_id: str):
+        """
+        Add a tool result message to the history.
+
+        This is useful for models (like OpenAI GPT) that support tool call messages,
+        where the assistant first emits a tool call and then receives a tool result
+        as a `role="tool"` message.
+
+        Both tool name and tool_call_id are required for proper attribution.
+
+        Args:
+            tool_name: Name of the tool/function that was executed (must be non-empty).
+            result: Serialized result returned by the tool (often JSON or text).
+            tool_call_id: tool_call_id from the preceding assistant tool call message (must be non-empty).
+        """
+        if not tool_name:
+            raise ValueError("tool_name is required for tool messages")
+        if not tool_call_id:
+            raise ValueError("tool_call_id is required for tool messages")
+
+        message = {
+            "role": "tool",
+            "name": tool_name,
+            "content": result,
+            "tool_call_id": tool_call_id,
+        }
+        self.history.append(message)
+
     # New generic methods for flexibility
     def add_message(self, role: str, content: str):
         """
@@ -358,3 +397,121 @@ class Context:
             The current llm provider.
         """
         return self.configuration.llm_provider
+
+    def _send_event_from_background(self, event: Event, name: str):
+        assert isinstance(event, Event), f"event is not of type Event: {type(event)}"
+
+        # Mark the event as background
+        event.background = True
+        event.background_task_name = name
+
+        # Send the event and a debug event
+        self.on_event(event)
+        self.on_debug_event(event, "output", self.serialize())
+
+    async def _wrap_handler(self, handler: Coroutine, name: str):
+        try:
+            # Check if the handler is an async generator meaning it will yield output events.
+            # If so, iterate over it directly and send the output events back over the WebSocket.
+            # Otherwise, just await the handler because there will not be any output events.
+            if hasattr(handler, "__aiter__"):
+                async for output_event in handler:
+                    self._send_event_from_background(output_event, name)
+            else:
+                await handler
+        except Exception as e:
+            error_details = format_error_with_filtered_traceback(e)
+            self._send_event_from_background(ErrorEvent(error_details), name)
+
+    def create_task(
+        self,
+        handler: Coroutine,
+        name: str = None,
+        interruptible: bool = False,
+        timeout: int = 30,
+    ):
+        if name in self.tasks:
+            self.cancel_task(name)
+
+        if not name:
+            name = str(uuid.uuid4())
+
+        task = asyncio.create_task(self._wrap_handler(handler, name))
+
+        self.tasks[name] = {
+            "task": task,
+            "timeout_task": asyncio.create_task(
+                asyncio.wait_for(task, timeout=timeout)
+            ),
+            "interruptible": interruptible,
+            "timeout": timeout,
+        }
+
+        return self.tasks[name]
+
+    def cancel_task(self, name: str):
+        if name in self.tasks:
+            # Cancel the main task if it is not done
+            if "task" in self.tasks[name] and not self.tasks[name]["task"].done():
+                self.tasks[name]["task"].cancel()
+
+            # Cancel the timeout task if it is not done
+            if (
+                "timeout_task" in self.tasks[name]
+                and not self.tasks[name]["timeout_task"].done()
+            ):
+                self.tasks[name]["timeout_task"].cancel()
+
+            del self.tasks[name]
+
+    def cancel_interruptible_tasks(self):
+        # Avoid RuntimeError by iterating over a static list of keys
+        names_to_cancel = [
+            name
+            for name, task in self.tasks.items()
+            if task.get("interruptible", False)
+        ]
+        for name in names_to_cancel:
+            self.cancel_task(name)
+
+    def cancel_all_tasks(self):
+        # Avoid RuntimeError by iterating over a static list of keys
+        names_to_cancel = list(self.tasks.keys())
+        for name in names_to_cancel:
+            self.cancel_task(name)
+
+    def has_unfinished_tasks(self):
+        for task_info in self.tasks.values():
+            task: asyncio.Task = task_info.get("task")
+            if task and not task.done():
+                return True
+
+        return False
+
+    async def wait_for_all_tasks(self):
+        # Collect all tasks that have not finished.
+        tasks_to_wait = []
+        for task_info in self.tasks.values():
+            task: asyncio.Task = task_info.get("task")
+            if task and not task.done():
+                tasks_to_wait.append(task)
+
+        async def exhaust_async_generator(gen):
+            try:
+                async for _ in gen:
+                    pass
+            except Exception:
+                pass
+
+        # Handle both regular tasks and async generators.
+        coros_to_wait = []
+        for t in tasks_to_wait:
+            # If the task is an async generator, exhaust it
+            if hasattr(t, "__aiter__") and hasattr(t, "__anext__"):
+                coros_to_wait.append(exhaust_async_generator(t))
+            else:
+                coros_to_wait.append(t)
+
+        # Wait for all tasks to complete.
+        if coros_to_wait:
+            await asyncio.gather(*coros_to_wait, return_exceptions=True)
