@@ -71,11 +71,30 @@ class StopEvent(Event):
     """Stops the current voice agent session. This will end the call.
 
     When emitted, this event terminates the active voice agent session
-    and ends the call connection.
+    and ends the call connection. Optionally plays a closing message
+    before ending the call.
+
+    Args:
+        closing_speech: Optional text to speak before ending the call
+        voice: Voice to use for closing speech (defaults to agent's default voice)
+        speed: Playback speed for closing speech (0.25x-4.0x, defaults to 1.0)
+        language: Language code for closing speech (defaults to "en")
     """
 
-    def __init__(self):
-        super().__init__("stop")
+    def __init__(
+        self,
+        closing_speech: str | None = None,
+        voice: str | None = None,
+        speed: float = 1.0,
+        language: str = "en",
+    ):
+        data = {
+            "closing_speech": closing_speech,
+            "voice": voice,
+            "speed": speed,
+            "language": language,
+        }
+        super().__init__("stop", data)
 
 
 class InterruptEvent(Event):
@@ -138,7 +157,7 @@ class TextToSpeechEvent(Event):
             - Cartesia (10 voices): brooke, david, sophie, luke, etc.
             Defaults to "nova"
         cache: Whether to cache the generated audio, defaults to True
-        interruptible: Whether the speech can be interrupted, defaults to True
+        interruptible: Whether the speech can be interrupted, defaults to None
         instructions: Optional voice styling instructions to control tone, emotion,
             or accent (e.g., 'enthusiastic with a Taiwanese accent').
             Supported by OpenAI voices only
@@ -154,7 +173,7 @@ class TextToSpeechEvent(Event):
         text: str,
         voice="nova",
         cache=True,
-        interruptible: bool = True,
+        interruptible: bool = None,
         instructions="",
         speed=1.0,
         language="en",
@@ -537,10 +556,17 @@ class InitializeEvent(Event):
         is_multifile: Whether the agent uses multiple files
         is_debug: Whether debug mode is enabled
         context: Initial context dictionary for the session
+        tracing_enabled: Whether tracing is enabled
     """
 
     def __init__(
-        self, code: str, hash: str, is_multifile: bool, is_debug: bool, context: dict
+        self,
+        code: str,
+        hash: str,
+        is_multifile: bool,
+        is_debug: bool,
+        context: dict,
+        tracing_enabled: bool,
     ):
         super().__init__(
             "initialize",
@@ -550,6 +576,7 @@ class InitializeEvent(Event):
                 "is_multifile": is_multifile,
                 "is_debug": is_debug,
                 "context": context,
+                "tracing_enabled": tracing_enabled,
             },
         )
 
@@ -697,6 +724,16 @@ class TransferSessionEvent(Event):
 
     For web/mic/api sessions:
       - if agent_id provided -> browser redirect to target agent/environment
+
+    Args:
+        phone_number: Phone number to transfer to (for phone sessions)
+        agent_id: Agent ID to transfer to (for web/mic/api sessions)
+        environment: Environment to transfer to
+        data: Additional data to pass to the transferred session
+        closing_speech: Optional text to speak before transferring
+        voice: Voice to use for closing speech (defaults to agent's default voice)
+        speed: Playback speed for closing speech (0.25x-4.0x, defaults to 1.0)
+        language: Language code for closing speech (defaults to "en")
     """
 
     def __init__(
@@ -706,12 +743,20 @@ class TransferSessionEvent(Event):
         agent_id: str | None = None,
         environment: str | None = None,
         data: dict | None = None,
+        closing_speech: str | None = None,
+        voice: str | None = None,
+        speed: float = 1.0,
+        language: str = "en",
     ):
         payload = {
             "phone_number": phone_number,
             "agent_id": agent_id,
             "environment": environment,
             "data": data or {},
+            "closing_speech": closing_speech,
+            "voice": voice,
+            "speed": speed,
+            "language": language,
         }
         super().__init__("transfer_session", payload)
 
@@ -736,6 +781,54 @@ class TranscriptPartEvent(Event):
         )
 
 
+class InputAllowedEvent(Event):
+    """Event to enable or disable user input.
+
+    InputAllowedEvent(allowed=false) disables user input until either
+    InputAllowedEvent(allowed=true) or HandlerEndEvent is received.
+
+    Args:
+        allowed: User input allowed
+    """
+
+    def __init__(self, allowed: bool):
+        super().__init__(
+            "input_allowed",
+            {
+                "allowed": allowed,
+            },
+        )
+
+
+class DTMFEvent(Event):
+    """Event for DTMF input.
+
+    This event is for internal use only. It is used to store DTMF input.
+    """
+
+    def __init__(self, digits: str):
+        super().__init__("dtmf", {"digits": digits})
+
+
+def event_to_dict(event: Event) -> dict:
+    """Convert an Event object to its dict representation.
+
+    Args:
+        event: The Event object to serialize
+
+    Returns:
+        Dict representation of the event
+    """
+    return {
+        "name": event.name,
+        "data": event.data,
+        "timestamp": event.timestamp,
+        "turn": event.turn,
+        "background": event.background,
+        "background_task_name": event.background_task_name,
+    }
+
+
 def event_to_str(event: Event) -> str:
     """Convert an Event object to its JSON string representation.
 
@@ -745,16 +838,7 @@ def event_to_str(event: Event) -> str:
     Returns:
         JSON string representation of the event
     """
-    return json.dumps(
-        {
-            "name": event.name,
-            "data": event.data,
-            "timestamp": event.timestamp,
-            "turn": event.turn,
-            "background": event.background,
-            "background_task_name": event.background_task_name,
-        }
-    )
+    return json.dumps(event_to_dict(event))
 
 
 def event_from_str(event_str: str) -> Event:
@@ -806,18 +890,25 @@ def event_from_str(event_str: str) -> Event:
         "log": lambda: LogEvent(data.get("message"), data.get("location")),
         "silence": lambda: SilenceEvent(data.get("duration")),
         "start": StartEvent,
-        "stop": StopEvent,
+        "stop": lambda: StopEvent(
+            closing_speech=data.get("closing_speech"),
+            voice=data.get("voice"),
+            speed=data.get("speed", 1.0),
+            language=data.get("language", "en"),
+        ),
         "text_to_speech": lambda: TextToSpeechEvent(
             data.get("text"),
             data.get("voice", "nova"),
             data.get("cache", True),
-            data.get("interruptible", True),
+            data.get("interruptible", None),
             data.get("instructions", ""),
             data.get("speed", 1.0),
             data.get("language", "en"),
             data.get("stream"),
         ),
-        "text": lambda: TextEvent(data.get("source"), data.get("text")),
+        "text": lambda: TextEvent(
+            data.get("source"), data.get("text"), data.get("language")
+        ),
         "timeout": lambda: TimeoutEvent(data.get("count"), data.get("ms_since_input")),
         "collect_payment": lambda: CollectPaymentEvent(data.get("amount")),
         "collect_payment_success": CollectPaymentSuccessEvent,
@@ -843,6 +934,7 @@ def event_from_str(event_str: str) -> Event:
             data.get("is_multifile"),
             data.get("is_debug"),
             data.get("context"),
+            data.get("tracing_enabled"),
         ),
         "handler_end": HandlerEndEvent,
         "debug": lambda: DebugEvent(
@@ -866,10 +958,16 @@ def event_from_str(event_str: str) -> Event:
             agent_id=data.get("agent_id"),
             environment=data.get("environment"),
             data=data.get("data", {}),
+            closing_speech=data.get("closing_speech"),
+            voice=data.get("voice"),
+            speed=data.get("speed", 1.0),
+            language=data.get("language", "en"),
         ),
         "transcript_part": lambda: TranscriptPartEvent(
             data.get("role"), data.get("content")
         ),
+        "input_allowed": lambda: InputAllowedEvent(data.get("allowed")),
+        "dtmf": lambda: DTMFEvent(data.get("digits")),
     }
 
     if name in event_types:
