@@ -138,11 +138,8 @@ def _update_sentence_buffer(
 
         # Check if we have a complete sentence (ends with punctuation)
         if any(punct in sentence_buffer for punct in punctuation_marks):
-            # Find the last sentence boundary
-            last_sentence_end = max(
-                (sentence_buffer.rfind(punct) for punct in punctuation_marks),
-                default=-1,
-            )
+            # Find the last sentence boundary, BUT exclude periods in time abbreviations
+            last_sentence_end = _find_last_sentence_boundary(sentence_buffer, punctuation_marks)
 
             if last_sentence_end != -1:
                 # Extract complete sentence
@@ -161,6 +158,146 @@ def _update_sentence_buffer(
     return sentence_buffer, None
 
 
+def _find_last_sentence_boundary(text, punctuation_marks):
+    """Find the last sentence boundary, excluding periods in common abbreviations like 'a.m.', 'p.m.', etc."""
+    last_sentence_end = -1
+    
+    for punct in punctuation_marks:
+        # Special handling for periods - skip those in abbreviations
+        if punct == '.':
+            # Collect all period positions, then filter from right to left
+            period_positions = [i for i, ch in enumerate(text) if ch == '.']
+            
+            # Check periods from right to left, skipping abbreviation periods
+            for i in reversed(period_positions):
+                # Look back to see if this period is part of an abbreviation
+                start_idx = max(0, i - 10)
+                substring = text[start_idx:i+1]
+                
+                # Only skip if this is an incomplete abbreviation (like "p." in "p.m" without the "m" yet)
+                # If period is at end of buffer with no following text, could be incomplete abbreviation
+                if i + 1 >= len(text):  # Period is at end of text
+                    # Check if it could be an incomplete abbreviation
+                    if _is_incomplete_abbreviation(substring):
+                        continue  # Skip this period, wait for more text
+                
+                # This is either a complete abbreviation or not an abbreviation - use it as boundary
+                # But still skip if it's recognized as a complete abbreviation in middle of sentence
+                if _is_abbreviation_period(substring):
+                    # At end of text - check if it's preceded by a number/time (like "6 p." or "3:30 p.")
+                    if i + 1 >= len(text):
+                        # Extract the last word before the period
+                        text_before = text[:i].strip()
+                        words = text_before.split()
+                        if words and len(words) >= 2:
+                            prev_word = words[-2]
+                            # If preceded by a number or time format, it's a complete time phrase
+                            # Patterns: "6", "3:30", "8:00", etc.
+                            if re.search(r'^\d+(?::\d+)?$', prev_word):
+                                return i  # Return the period position
+                        continue  # Skip, might have more content (incomplete abbreviation)
+                    # In middle of text - only skip if no space after (embedded in word)
+                    if text[i + 1] not in (' ', '\n', '\t'):
+                        continue  # Part of longer word/phrase, skip
+                    # Has space after - it's a sentence boundary
+                
+                return i
+        else:
+            # For other punctuation, just find the last occurrence
+            pos = text.rfind(punct)
+            if pos > last_sentence_end:
+                last_sentence_end = pos
+    
+    # If we found a non-period punctuation, return it
+    if last_sentence_end != -1:
+        return last_sentence_end
+    
+    # No sentence boundary found
+    return -1
+
+
+def _is_abbreviation_period(text):
+    """Check if the period at the end of text is part of a known abbreviation."""
+    text = text.strip()
+    if not text or not text.endswith('.'):
+        return False
+    
+    # Remove the trailing period for analysis
+    prefix = text[:-1]
+    
+    # For abbreviations to be skipped, they should be at the end of text with space/word boundary before
+    # Get the last "word" (sequence of chars after the last space or from start)
+    words = prefix.split()
+    if not words:
+        return False
+    
+    last_word = words[-1]
+    
+    # Time abbreviations: a.m, p.m, a., p. (also incomplete forms like "p." in streaming)
+    if re.search(r'^[ap](?:\.|\.m)?$', last_word, re.IGNORECASE):
+        return True
+    
+    # Titles and suffixes (must be standalone, not part of longer word)
+    if re.search(r'^(Mr|Mrs|Ms|Dr|Prof|St|Rev|Sr|Jr)$', last_word, re.IGNORECASE):
+        return True
+    
+    # Company suffixes
+    if re.search(r'^(Inc|Ltd|Co|Corp)$', last_word, re.IGNORECASE):
+        return True
+    
+    # Common abbreviations
+    if re.search(r'^(etc|vs|e\.g|i\.e|et al)$', last_word, re.IGNORECASE):
+        return True
+    
+    return False
+
+
+def _is_incomplete_abbreviation(text):
+    """Check if text ends with an incomplete abbreviation pattern (e.g., 'p.' but no 'm' following).
+    
+    Only return True if the pattern strongly suggests incompleteness.
+    For "6 p.", it's likely a complete sentence with time, not an incomplete "p.m",
+    so we should NOT treat it as incomplete.
+    """
+    text = text.strip()
+    if not text or not text.endswith('.'):
+        return False
+    
+    prefix = text[:-1]
+    words = prefix.split()
+    if not words:
+        return False
+    
+    last_word = words[-1]
+    
+    # Only treat as incomplete if we have a VERY strong signal that more is coming
+    # For example, at the very start of a number/sentence context with nothing before
+    # But NOT for "6 p." which is a complete time phrase
+    
+    # Pattern: word that is just a single letter followed by period
+    # BUT only if the word before it is also a single letter or abbreviation
+    # Examples: "Mr." by itself is incomplete, "6 p." is complete
+    if re.search(r'^[ap]$', last_word, re.IGNORECASE):
+        # Check if there's a word before this single letter
+        if len(words) >= 2:
+            prev_word = words[-2]
+            # If previous word is a number like "6", this is likely "6 p.m" (complete time phrase)
+            # Treat as complete, not incomplete
+            if re.search(r'^\d+$', prev_word):
+                return False  # "6 p." is complete
+            # If previous word is another single letter/abbrev, might be incomplete
+            if len(prev_word) == 1:
+                return True  # "a p." might be incomplete
+        
+        # Single letter alone with no context - ambiguous, but safer to treat as incomplete
+        # (could be waiting for "m" in "p.m")
+        # Actually, if it's at end of sentence naturally, let's treat as complete
+        # The logic in _find_last_sentence_boundary will handle it with context
+        return False  # Be permissive - let the context decide
+    
+    return False
+
+
 def _clean_text_for_speech(text):
     """
     Clean text for better speech synthesis by removing/replacing problematic characters,
@@ -237,4 +374,4 @@ def _clean_text_for_speech(text):
     if not re.search(r"[\w\u4e00-\u9fff\u3400-\u4dbf\u20000-\u2a6df\u2a700-\u2b73f\u2b740-\u2b81f\u2b820-\u2ceaf\uf900-\ufaff\u3300-\u33ff\ufe30-\ufe4f\u0900-\u097f\u0980-\u09ff\u0a00-\u0a7f\u0a80-\u0aff\u0b00-\u0b7f\u0b80-\u0bff\u0c00-\u0c7f\u0c80-\u0cff\u0d00-\u0d7f\u0d80-\u0dff\u0e00-\u0e7f\u0e80-\u0eff\u0f00-\u0fff]", text):
         return ""
 
-    return text
+    return text
\ No newline at end of file
